unit CGFUnit;

{This unit contains functions used by this
 program that are not members of any classes
 in it
 CGFUnit --> Code Generator Function Unit}

interface
  uses
    SysUtils, ShellAPI, Dialogs, Windows,
    Controls;

  procedure UpperChar (var ch : char);
  function CorrectCase (const s : string) : string;
  function IntToHTMLHex (Value, Digits : integer) : string;
  function ShellExe (fName : TFileName) : integer;
  function LwrChr (i : integer) : char;
  function CanWrite (fName : TFileName) : boolean;

implementation

procedure UpperChar (var ch : char);
begin
  {if ch is lower case then convert it to
   upper case}
  if (ch >= 'a') and (ch <= 'z') then
    ch := Chr (Ord (ch) + Ord ('A') - Ord ('a'));
end;

function CorrectCase (const s : string) : string;
var i : cardinal;
begin
  {This function makes a string lowercase
   except for the first letter and every letter
   after a space, tab, or carriage return}

  result := LowerCase (s);
  i := length (s);

  {This is here because the while loop below
   does not cover the 1st character in result}
  if i > 0 then
    UpperChar (result[1]);

  {Goes through every letter but 1st one, and
   if the one before it is not a letter or
   number, then it is capitolized}
  while i > 0 do begin
    if not (result[i - 1] in ['a'..'z', 'A'..'Z', '0'..'9']) then
      UpperChar (result [i]);

    dec (i);
  end;
end;

function IntToHTMLHex (Value, Digits : integer) : string;
begin
  result := IntToHex (Value, Digits);

  {html has BGR but Delphi has RBG so the
   first and last two characters need to be
   swapped}

  // Add a copy of 1st 2 chars to end
  result := result + copy (result, 1, 2);

  // Set 1st 2 chars to what last 2 chars were
  result[1] := result[Digits - 1];
  result[2] := result[Digits];

  // Delete what was the last 2 chars
  delete (result, Digits - 1, 2);
end;

function ShellExe (fName : TFileName) : integer;
begin
  {Implements the ShellExecute function
   without having to worry about the
   parameters that are the same most of the
   time anyway; also generates appropriate
   error messages}
  result := ShellExecute (0, Nil, PChar (fName), Nil, Nil, SW_ShowDefault);
  case result of
    SE_ERR_FNF : ShowMessage ('File "' + fName + '" not found');
    SE_ERR_PNF : ShowMessage ('Path "' + ExtractFilePath (fName) + '" not found');
    SE_ERR_ACCESSDENIED : ShowMessage ('Access to "' + fName + '" denied');
    SE_ERR_OOM : ShowMessage ('Out of memory');
    SE_ERR_DLLNOTFOUND : ShowMessage ('DLL not found');
  end;
end;

function LwrChr (i : integer) : char;
begin
  {converts i to a character, then to the
   lower case of that character}
  result := Chr (i);
  if result in ['A'..'Z'] then
    result := Chr (i + Ord ('a') - Ord ('A'));
end;

function CanWrite (fName : TFileName) : boolean;
begin
  {returns true is a file doesn't exists or
   if it exists and permission is give to
   overwrite it}
  if FileExists (fName) then
    result := MessageDlg ('file "' + fName + '" exists. Overwright?',
                   mtWarning, [mbYes, mbNo], 0) = mrYes
  else result := true;
end;

end.

