unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ExtCtrls, Clipbrd;

const
  // names of files used in this program
  InIFileName = 'codegen.ini';
  HelpFileName = 'help.htm';
  TempFileName = 'temp.htm';

  // number of digits Hex numbers will have
  HexLen = 6;

  // title of the application
  ProgName = 'Color Code Generator 1.1';

  // indexes for SpacingRG
  siSingle = 0; // si --> Spacing Index
  siDouble = 1;

  // indexes for CasingRG
  ciLower = 0; // cs --> Casing Index
  ciUpper = 1;
  ci1stLetterUpper = 2;

  // carriage return
  CR = #13#10;

type
  TMainForm = class(TForm)
    ColorDialog1: TColorDialog;
    SaveDialog1: TSaveDialog;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    SpinEdit1: TSpinEdit;
    SpacingRG: TRadioGroup;
    GroupBox2: TGroupBox;
    ColorNumLab: TLabel;
    ListBox1: TListBox;
    AddColorBtn: TButton;
    DeleteBtn: TButton;
    GroupBox3: TGroupBox;
    Memo1: TMemo;
    GroupBox4: TGroupBox;
    SaveBtn: TButton;
    TestBtn: TButton;
    CopyBtn: TButton;
    InsertBtn: TButton;
    ProgramGB: TGroupBox;
    CloseBtn: TButton;
    HelpBtn: TButton;
    AboutBtn: TButton;
    CasingRG: TRadioGroup;
    procedure AddColorBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TestBtnClick(Sender: TObject);
    procedure CopyBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure SpinEdit1KeyPress(Sender: TObject; var Key: Char);
    procedure SpinEdit1Exit(Sender: TObject);
    procedure ListBox1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure HelpBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
  private
    { Private declarations }
    Modified : boolean;
    SourceDir : TFileName;
    function ChangeCase (const s : string) : string;
    function GetFirstSelected : integer;
    procedure SaveOpt;
    procedure EnableMake;
    procedure EnableDelete;
    procedure DrawLab;
    procedure Init;
    procedure MakeFile (fName : TFileName);
    function MemoToCode : string;
    function GetNum (i, MemoLen : cardinal) : string;
    function ModifyExt (fName : TFileName) : TFileName;
    function EnoughInfo : boolean;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses AboutUnit, CGFUnit;

{$R *.DFM}

function TMainForm.ChangeCase (const s : string) : string;
begin
  {returns s as the correct case according
   to the settings of CasingRG}
  case CasingRG.ItemIndex of
    ciLower : result := LowerCase (s);
    ciUpper : result := UpperCase (s);
    else result := CorrectCase (s);
  end; // case
end;

function TMainForm.GetFirstSelected : integer;
var i : cardinal;
begin
  for i := 0 to ListBox1.Items.Count - 1 do
    if ListBox1.Selected[i] then begin
      result := i;
      exit;
    end;

  result := -1;
end;

procedure TMainForm.SaveOpt;
var
  FileVar : TextFile;
  i : cardinal;
begin
  {Saves all the options to a file when the
   program ends}
  AssignFile (FileVar, SourceDir + InIFileName);
  Rewrite (FileVar);
    WriteLn (FileVar, ExtractFilePath (SaveDialog1.FileName));
    WriteLn (FileVar, SpacingRG.ItemIndex);
    WriteLn (FileVar, CasingRG.ItemIndex);
    WriteLn (FileVar, SpinEdit1.Value);

    with ColorDialog1.CustomColors do begin
      WriteLn (FileVar, Count);

      if Count > 0 then
        for i := 0 to Count - 1 do
          WriteLn (FileVar, Strings[i]);
    end;
  CloseFile (FileVar);
end;

procedure TMainForm.EnableMake;
var b : boolean;
begin
  {Appropiately enables the Save, Copy, and
   Test buttons when there is text in the
   memo box and enough colors to do something}
  b := EnoughInfo;
  SaveBtn.Enabled := b;
  CopyBtn.Enabled := b;
  TestBtn.Enabled := b;
end;

procedure TMainForm.EnableDelete;
begin
  {Enables the delete and insert buttons when
   something is selected}
  DeleteBtn.Enabled := ListBox1.SelCount > 0;
  InsertBtn.Enabled := ListBox1.SelCount > 0;
end;

procedure TMainForm.DrawLab;
begin
  ColorNumLab.Caption := '(' + IntToStr (ListBox1.Items.Count) + ') Co&lors';
end;

procedure TMainForm.AddColorBtnClick(Sender: TObject);
var s : string;
begin
  if not ColorDialog1.Execute then exit;

  s := IntToHTMLHex (ColorDialog1.Color, HexLen);

  ListBox1.Items.Add (s);

  Modified := true;
  EnableMake;
  DrawLab;
end;

procedure TMainForm.Init;
var
  FileVar : TextFile;
  s : string;
  i : integer;
begin
  {Reads options that were saved to a file}
  if not FileExists (SourceDir + InIFileName) then exit;
  AssignFile (FileVar, SourceDir + InIFileName);
  Reset (FileVar);
    ReadLn (FileVar, s);
    SaveDialog1.InitialDir := s;

    ReadLn (FileVar, i);
    SpacingRG.ItemIndex := i;

    ReadLn (FileVar, i);
    CasingRG.ItemIndex := i;

    ReadLn (FileVar, i);
    SpinEdit1.Value := i;

    ReadLn (FileVar, i);
    if i > 0 then
      with ColorDialog1.CustomColors do
        repeat
          ReadLn (FileVar, s);
          Add (s);
          dec (i);
        until i = 0;

  CloseFile (FileVar);
end;

procedure TMainForm.MakeFile (fName : TFileName);
var
  FileVar : TextFile;
  LineBreak : string;
begin
  {Saves generated code to a file; it writes
   in <html><body> and their end tags in
   addition to the generated code; the non
   generated code it formatted like the
   generated code}

  if SpacingRG.ItemIndex = siSingle then
    LineBreak := CR
  else
    LineBreak := CR + CR;

  AssignFile (FileVar, fName);
  Rewrite (FileVar);
    WriteLn (FileVar, ChangeCase ('<html>' + LineBreak + '<body>'));

    if SpacingRG.ItemIndex = siDouble then
      WriteLn (FileVar);

    WriteLn (FileVar, MemoToCode);

    if SpacingRG.ItemIndex = siDouble then
      WriteLn (FileVar);

    WriteLn (FileVar, ChangeCase ('</body>' + LineBreak + '</html>'));
  CloseFile (FileVar);
end;

function TMainForm.MemoToCode : string;
var
  s1, s2, indent, cr : string;
  i, i2, MemoLen : cardinal;
  AddTag : boolean;
begin
  {Returns a string containing the generated
   code}

  {Change the cursor to an HourGlass because
   sometimes this function can be a lengthy
   operation}
  Screen.Cursor := crHourGlass;

  if SpacingRG.ItemIndex = siSingle then
    cr := #13#10 //single spaced
  else
    cr := #13#10#13#10; //double spaced

  if SpinEdit1.Value > 0 then
    for i := 1 to SpinEdit1.Value do
      indent := indent + #32;

  result := indent;

  {A function will be called to generate a
   color as a string; s1 will go in front of
   it and s2 will go behing it}
  s1 := ChangeCase ('<font color = #');
  s2 := ChangeCase ('</font>');

  {MemoLen is the number of visible characters
   in Memo1; adding spaces to text shouldn't
   change the color the visible characters
   will have; HTML tags are also not visible
   characters}
  MemoLen := Memo1.GetTextLen - 1;
  i2 := 1;
  with Memo1 do
    while i2 <= GetTextLen do begin
      {subtracts 1 for MemoLen for every
       character in Memo1.Text that is an
       HTML tag}
      if Text[i2] = '<' then begin
        dec (MemoLen);

        while (i2 < GetTextLen)
        and (Text[i2] <> '>') do begin
          inc (i2);
          dec (MemoLen);
        end;
      end;

    if Memo1.Text[i2] in [#32, #9, #13, #10] then
      dec (MemoLen);

    inc (i2);
  end;

  AddTag := false;
  i2 := 0;
  {i2 will be used a the number of visible
   characters that have been transversed}
  with Memo1 do
    for i := 1 to GetTextLen do begin
      {Ads an HTML tag; it adds characters to
       to result until > is encountered
       changing AddTag to false}
      if AddTag then begin
        result := result + text[i];
        if text[i] = '>' then AddTag := false;
      end else
        case text[i] of
          '<' : begin
             result := result + '<';
             AddTag := true;
          end;

          {these characters cause a new line of
           code to be started; even w/o
           indentation, in HTML the new line
           will create a space; the if condences
           many spaces, tabs, etc into one}
          #32, #13, #9 :
             if i > 1 then
               if not (text[i - 1] in [#32, #10, #10, #9]) then begin
                 result := result +
                 cr + indent;
               end;

          {#10 occurs in carriage returns after
           #13; doesn't need to need carriage
           return and indent on #10 since
           it had on #13, and #10 shouldn't be
           added in a font tag; in fact, since
           it is not a visible character, it
           would cause i2 to eventually become
           > MemoLen and cause index + 1
           (in GetNum) to go out of bounds}
          #10 : ;

          {creates a font tag for a character
           in Memo1}
          else begin
            result := result + s1 + GetNum (i2, MemoLen) + '>' + text[i] + s2;
            inc (i2);
          end;
        end;//case...

    end;//for...

  Screen.Cursor := crDefault;
end;

function TMainForm.GetNum (i, MemoLen : cardinal) : string;
var
  index : byte;
  j1, j2 : byte;
  i1, i2, i3 : byte;
  frac : single;
begin
  {Calculates the Hexadecimal number based on
   Chosen color, length of visible string, and
   position in that string and returns it
   as a string to the MemoToCode function}

  {End of characters so return last color;
   prevents index + 1 = ListBox...Count (later)
   which would make ListBox.Items[index] out
   of array bounds}
  if i = MemoLen then begin
    result := IntToHex (StrToInt ('$' + ListBox1.Items[ListBox1.Items.Count - 1]), 6);
    exit;
  end;

  {index and index + 1 are the indexes in
   ListBox1 of the colors that are currently
   being faded between}
  index := trunc (i / MemoLen * (ListBox1.Items.Count - 1));

  {i is the number of visible characters since
   the start of fading from one color to
   enother}
  i := i - index * MemoLen div (ListBox1.Items.Count - 1);

  {frac represents the fraction from one color to another,
   so 0.5 means half way between colors;
   MemoLen / (ListBox1.Items.Count - 1) is the number of
   characters b/t any two colors, so i / that is frac}
  frac :=  i / MemoLen * (ListBox1.Items.Count - 1);
  if frac > 1 then frac := 1;

  {B, G, and R of and BGR color must be
   averaged (weighted avg) separately to
   correctly average (weighted) two BGR colors;
   i1 is the B, i2 the G, and i3 the R; j1
   and j2 are the B (a few lines later the
   G, then the R) of the first and second
   color, respectively}
  j1 := StrToInt ('$' + copy (ListBox1.Items[index], 1, 2));
  j2 := StrToInt ('$' + copy (ListBox1.Items[index + 1], 1, 2));
  i1 := j1 + round ((j2 - j1) * frac);

  j1 := StrToInt ('$' + copy (ListBox1.Items[index], 3, 2));
  j2 := StrToInt ('$' + copy (ListBox1.Items[index + 1], 3, 2));
  i2 := j1 + round ((j2 - j1) * frac);

  j1 := StrToInt ('$' + copy (ListBox1.Items[index], 5, 2));
  j2 := StrToInt ('$' + copy (ListBox1.Items[index + 1], 5, 2));
  i3 := j1 + round ((j2 - j1) * frac);

  result := IntToHex (i1, 2) + IntToHex (i2, 2) + IntToHex (i3, 2);
end;

function TMainForm.ModifyExt (fName : TFileName) : TFileName;
var ext : TFileName;
begin
  {If extension is not .html, .htm, or .txt,
   then .html is added}
  ext := ExtractFileExt (fName);
  if (StrIComp (PChar (ext), '.htm') = 0)
  or (StrIComp (PChar (ext), '.html') = 0)
  or (StrIComp (PChar (ext), '.txt') = 0) then
    result := fName
  else
    result := fName + '.html';
end;

procedure TMainForm.SaveBtnClick(Sender: TObject);
begin
  if not SaveDialog1.Execute then exit;

  SaveDialog1.FileName := ModifyExt (SaveDialog1.FileName);

  if not CanWrite (SaveDialog1.FileName) then exit;

  MakeFile (SaveDialog1.FileName);
  Modified := false;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // Initialize variables
  Modified := false;
  DrawLab;
  EnableMake;
  EnableDelete;

  Application.Title := ProgName;
  Caption := ProgName;

  SourceDir := ExtractFilePath (Application.ExeName);
  Init; // get saved options
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  {Prompt to not exit if code hasn't been
   saved or copied}
  if Modified and EnoughInfo then
    if MessageDlg ('Do you really want to exit without saving or copying your code?',
                mtConfirmation, [mbYes, mbNo], 0) = mrNo then begin
    Action := caNone;
    exit;
  end;

  SaveOpt; // save options
end;

procedure TMainForm.TestBtnClick(Sender: TObject);
var fName : TFileName;
begin
  // make temporary file name
  fName := SourceDir + TempFileName;
  MakeFile (fName); // save that file
  ShellExe (fName); // execute that file
end;

procedure TMainForm.CopyBtnClick(Sender: TObject);
begin
  Modified := false;
  Clipboard.AsText := MemoToCode;
end;

procedure TMainForm.DeleteBtnClick(Sender: TObject);
var i : byte;
begin
  // Deletes all selected colors in ListBox1
  {A for loop is not used because it would go
   from 0 to List...Count whici changes}
  i := 0;
  while i < ListBox1.Items.Count do
    if ListBox1.Selected[i] then
      ListBox1.Items.Delete (i)
    else
      inc (i);

  EnableMake;
  EnableDelete;
  DrawLab;
end;

procedure TMainForm.ListBox1Click(Sender: TObject);
begin
  EnableDelete;
end;

procedure TMainForm.Memo1Change(Sender: TObject);
begin
  Modified := true;
  EnableMake;
end;

procedure TMainForm.SpinEdit1KeyPress(Sender: TObject; var Key: Char);
begin
  {Need a number here so don't let the user
   write anything else (except a backspace)}
  if not (Key in ['0'..'9', #8]) then Key := #0;
end;

procedure TMainForm.SpinEdit1Exit(Sender: TObject);
begin
  {If everyting was erased by backspacing,
   set Value to 0}
  if SpinEdit1.GetTextLen = 0 then
    SpinEdit1.Value := 0;
end;

procedure TMainForm.ListBox1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {lets one perform DeleteBtn Action with
   the Delete Key}
  if Shift <> [] then exit;
  if Key = VK_DELETE then DeleteBtn.Click;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {GroupBox control has not FocusControl
   propoerty so this compensates}
  if (ssAlt in Shift)
  and (LwrChr (Key) = 'x') then
    Memo1.SetFocus;

  //F1 to call Help menu
  if Key = VK_F1 then HelpBtn.Click;
end;

procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then Close;
end;

procedure TMainForm.HelpBtnClick(Sender: TObject);
begin
  ShellExe (SourceDir + HelpFileName);
end;

procedure TMainForm.AboutBtnClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

function TMainForm.EnoughInfo : boolean;
begin
  {returns if there is enough information for
   code to be generated}
  result := (Memo1.GetTextLen > 0)
    and (ListBox1.Items.Count >= 2);
end;

procedure TMainForm.InsertBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  i := GetFirstSelected;
  if i < 0 then exit;

  if not ColorDialog1.Execute then exit;

  s := IntToHTMLHex (ColorDialog1.Color, HexLen);

  ListBox1.Items.Insert (i, s);

  Modified := true;
  EnableMake;
  DrawLab;
end;

procedure TMainForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

end.

